//+---------------------------------------------------------------------+ 
//|                                                 Keltner_Channel.mq5 | 
//|                                Copyright  2011,   Nikolay Kositsin | 
//|                                 Khabarovsk,   farria@mail.redcom.ru | 
//+---------------------------------------------------------------------+ 
//|     SmoothAlgorithms.mqh      |
//|   (): __\\MQL5\Include        |
//+---------------------------------------------------------------------+
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link "farria@mail.redcom.ru"
//---   
#property version   "1.01"
//---     
#property indicator_chart_window 
//---   
#property indicator_buffers 3 
//---    
#property indicator_plots   3
//+-----------------------------------+
//|       |
//+-----------------------------------+
//---     
#property indicator_type1   DRAW_LINE
//---        
#property indicator_color1 clrYellow
//---   -  
#property indicator_style1  STYLE_SOLID
//---     1
#property indicator_width1  1
//---   
#property indicator_label1  "Upper Keltner"
//+-----------------------------------+
//|       |
//+-----------------------------------+
//---     
#property indicator_type2   DRAW_LINE
//---        
#property indicator_color2 clrGray
//---   -  
#property indicator_style2  STYLE_SOLID
//---     1
#property indicator_width2  1
//---   
#property indicator_label2  "Middle Keltner"
//+-----------------------------------+
//|       |
//+-----------------------------------+
//---     
#property indicator_type3   DRAW_LINE
//---        
#property indicator_color3 clrMagenta
//---   -  
#property indicator_style3  STYLE_SOLID
//---     1
#property indicator_width3  1
//---   
#property indicator_label3  "Lower Keltner"
//+-----------------------------------+
//|             |
//+-----------------------------------+
enum Applied_price_      //  
  {
   PRICE_CLOSE_ = 1,     // Close
   PRICE_OPEN_,          // Open
   PRICE_HIGH_,          // High
   PRICE_LOW_,           // Low
   PRICE_MEDIAN_,        // Median Price (HL/2)
   PRICE_TYPICAL_,       // Typical Price (HLC/3)
   PRICE_WEIGHTED_,      // Weighted Close (HLCC/4)
   PRICE_SIMPL_,         // Simple Price (OC/2)
   PRICE_QUARTER_,       // Quarted Price (HLOC/4) 
   PRICE_TRENDFOLLOW0_,  // TrendFollow_1 Price 
   PRICE_TRENDFOLLOW1_,  // TrendFollow_2 Price 
   PRICE_DEMARK_         // Demark Price
  };
//+-----------------------------------+
//|         |
//+-----------------------------------+
input uint KeltnerPeriod=20;               //  
input ENUM_MA_METHOD MA_Method_=MODE_SMA;  //  
input double Ratio = 1.0;                  //  
input Applied_price_ IPC=PRICE_CLOSE_;     //  
input int Shift=0;                         //      
//+-----------------------------------+
//---  
double UpperBuffer[];
double MiddleBuffer[];
double LowerBuffer[];
//---      
int min_rates_total;
//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+ 
#include <SmoothAlgorithms.mqh>
//+------------------------------------------------------------------+
//|                            |
//+------------------------------------------------------------------+ 
double GetKeltner(int period,int bar,const double &High[],const double &Low[])
  {
//---
   double keltner,sum=0;
   for(int iii=0; iii<period; iii++) sum+=High[bar-iii]-Low[bar-iii];
   keltner=sum/period;
//---
   return(keltner);
  }
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+    
void IndInit(int Number,double &Buffer[],double Empty_Value,int Draw_Begin,int nShift)
  {
//---      
   SetIndexBuffer(Number,Buffer,INDICATOR_DATA);
//---      
   PlotIndexSetInteger(Number,PLOT_DRAW_BEGIN,Draw_Begin);
//---   ,      
   PlotIndexSetDouble(Number,PLOT_EMPTY_VALUE,Empty_Value);
//---    2    Shift
   PlotIndexSetInteger(Number,PLOT_SHIFT,nShift);
//---
  }
//+------------------------------------------------------------------+    
//| Keltner Channel indicator initialization function                | 
//+------------------------------------------------------------------+  
void OnInit()
  {
//---  
   min_rates_total=int(2*KeltnerPeriod+1);
//---   
   IndInit(0,UpperBuffer,0.0,min_rates_total,Shift);
   IndInit(1,MiddleBuffer,0.0,min_rates_total,Shift);
   IndInit(2,LowerBuffer,0.0,min_rates_total,Shift);
//---      
   string shortname;
   StringConcatenate(shortname,"Keltner( KeltnerPeriod = ",KeltnerPeriod,")");
//---           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);
//---     
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
//---  
  }
//+------------------------------------------------------------------+  
//| Keltner Channel iteration function                               | 
//+------------------------------------------------------------------+  
int OnCalculate(const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---       
   if(rates_total<min_rates_total) return(0);
//---       
   double price_,middle,Keltner;
//---   
   int first1,first2,bar;
//---    first    
   if(prev_calculated==0) //      
     {
      first1=0; //      
      first2=min_rates_total;
     }
   else //      
     {
      first1=prev_calculated-1;
      first2=first1;
     }
//---    Moving_Average  StdDeviation
   static CMoving_Average MA;
//---      
   for(bar=first1; bar<rates_total; bar++)
     {
      //---    PriceSeries     Series
      price_=PriceSeries(IPC,bar,open,low,high,close);
      MiddleBuffer[bar]=MA.MASeries(0,prev_calculated,rates_total,KeltnerPeriod,MA_Method_,price_,bar,false);
     }
//---     
   for(bar=first2; bar<rates_total; bar++)
     {
      middle=MiddleBuffer[bar];
      Keltner=Ratio*GetKeltner(KeltnerPeriod,bar,high,low);
      UpperBuffer[bar]=middle+Keltner;
      LowerBuffer[bar]=middle-Keltner;
     }
//---    
   return(rates_total);
  }
//+------------------------------------------------------------------+
